/*
 * Copyright (C) 2013 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Raúl Yeguas <neokore@gmail.com>
 *              Martin Borho <martin@borho.net>
 */
import QtQuick 2.0
import Ubuntu.Components 0.1
import "./" as Components

Rectangle {
    id: currentWeather
    color: "transparent"

    // For Status I take the same condition codes from OpenWeatherMap (http://openweathermap.org/wiki/API/Weather_Condition_Codes)
    property int condition
    property string icon
    property int currentTemp
    property int minTemp
    property int maxTemp
    property bool showMaxTemp
    property int windSpeed
    property string windDir
    property int humidity
    property real precipitation
    property int pressure
    property variant hourly
    property string tempScale: (mainView.settings["units"] === "imperial") ? "F" : "C"
    property string speedScale: (mainView.settings["wind_units"] === "mph") ? "mph" : "km/h"
    property string precipScale: (mainView.settings["precip_units"] === "in") ? "in" : "mm"

    property NumberAnimation appearAnimation: currentConditionIcon.appearAnimation

    width: parent.width
    anchors.fill: parent
    anchors.horizontalCenter: parent.horizontalCenter

    Flipable {
            id: flipable
            height: parent.height
            width:parent.width
            anchors.verticalCenter: parent.verticalCenter
            anchors.verticalCenterOffset: -units.gu(3)
            property bool flipped: false

            MouseArea {
                id: flippedarea
                property bool wasHourlyScrolled: false
                property int startY: 0
                property double startTime: 0
                property int sectionHeight: parent.height/hourly.count
                property int lastIndex: -1
                // without tab header!
                height: parent.height-units.gu(9.5)
                width:parent.width
                anchors.bottom: parent.bottom
                anchors.bottomMargin: units.gu(2)
                onClicked: {
                    mouse.accepted = true;
                    if(!wasHourlyScrolled) {
                        var movement = Math.round(startY-mouse.y);
                        if(movement < 10 && movement > -10) {
                            // flip to detail data
                            flipable.flipped = !flipable.flipped;
                        } else {
                            // switch to next/prevoius day
                            if(movement >= 10) {
                                dailyForecastList.incrementCurrentIndex();
                            } else if(movement <= -10) {
                                dailyForecastList.decrementCurrentIndex();
                            }
                            dailyForecastList.positionViewAtIndex(dailyForecastList.currentIndex,ListView.SnapPosition);
                            flipable.flipped = false;
                        }
                    }
                }
                onPressed: {
                    wasHourlyScrolled = false;
                    startY = mouse.y;
                    startTime = new Date().getTime()
                }
                onPositionChanged: {
                    mouse.accepted = true;
                    var distance = (parent.height - mouse.y),
                        now = new Date().getTime(),
                        speed = distance/(now-startTime);
                    if(speed < 2.3 && !flipable.flipped) {
                        // slow scrolling detected
                        var movement = Math.round(distance/sectionHeight);
                        if(movement > 0) {
                            var hourIndex = movement-1,
                                currentHour = currentWeather.hourly.get(hourIndex);
                            if( currentHour !== undefined && (lastIndex < 0 || lastIndex !== hourIndex)) {
                                wasHourlyScrolled = true;
                                currentConditionIcon.condition = currentWeather.hourly.get(hourIndex).icon;
                                var temp = currentWeather.hourly.get(hourIndex)[mainView.settings["units"]].temp;
                                currentConditionTempContent.currentTemp = temp;
                                adjustBackground(Math.round(temp));
                                var dateData =  currentWeather.hourly.get(hourIndex).date;
                                var dateTime = new Date(dateData.year, dateData.month, dateData.date, dateData.hours, dateData.minutes)
                                // TRANSLATORS: this is a date to which a time of the day will be appended
                                var dateTimeString = Qt.formatDateTime(dateTime, i18n.tr('dddd, dd MMMM')) + ' - ' + Qt.formatTime(dateTime);
                                dateComponent.dateString = dateTimeString;
                                dateComponent.renderText();                                
                                hourlyScrollEffect.start();
                                lastIndex = hourIndex;
                            }
                        }
                    }
                }
                onReleased: {
                    if(wasHourlyScrolled) {
                        // reset temp and date to current day
                        var currentDayData = dayForecastModel.get(dailyForecastList.currentIndex);
                        currentConditionIcon.condition = currentDayData.condIcon;
                        currentConditionIcon.appearAnimation.start();
                        currentConditionTempContent.currentTemp = currentDayData.temp;
                        adjustBackground(Math.round(currentDayData.temp));
                        dateComponent.dateString = currentDayData.date;
                        dateComponent.renderText();
                        lastIndex = -1;
                    }
                }
            }

            back:
                Rectangle {
                    id: backrect
                    anchors.fill: parent
                    color: "transparent"

                    Column {
                        id: leftColumn
                        anchors {
                            horizontalCenter: parent.horizontalCenter
                            horizontalCenterOffset: units.gu(-8)
                            verticalCenter: parent.verticalCenter
                        }
                        spacing: units.gu(1)

                        UbuntuShape {
                            color: "#10000000"
                            radius: "medium"

                            width: units.gu(15)
                            height: units.gu(15)

                            Components.WeatherDetailComponent {
                                value: currentWeather.humidity
                                measure: i18n.tr("Humidity")
                                unit: i18n.tr('%')
                                anchors.fill: parent
                            }
                        }

                        UbuntuShape {
                            color: "#10000000"
                            radius: "medium"

                            width: units.gu(15)
                            height: units.gu(15)

                            Components.WeatherDetailComponent {
                                objectName: "WindSpeedValue"
                                value: currentWeather.windSpeed
                                measure: i18n.tr("Wind speed")
                                unit: currentWeather.speedScale + " " + currentWeather.windDir
                                anchors.fill: parent
                            }
                        }
                    }

                    Column {
                        id: rightColumn
                        anchors {
                            horizontalCenter: parent.horizontalCenter
                            horizontalCenterOffset: units.gu(8)
                            verticalCenter: parent.verticalCenter
                        }
                        spacing: units.gu(1)

                        UbuntuShape {
                            color: "#10000000"
                            radius: "medium"

                            width: units.gu(15)
                            height: units.gu(15)

                            Components.WeatherDetailComponent {
                                value: currentWeather.pressure
                                measure: i18n.tr("Pressure")
                                unit: i18n.tr("mbar")
                                anchors.fill: parent
                            }
                        }

                        UbuntuShape {
                            color: "#10000000"
                            radius: "medium"

                            width: units.gu(15)
                            height: units.gu(15)

                            Components.WeatherDetailComponent {
                                objectName: "PrecipitationValue"
                                value: currentWeather.precipitation
                                measure: i18n.tr("Precipitation")
                                unit: currentWeather.precipScale
                                anchors.fill: parent
                            }
                        }
                    }
                }

            front:
                Rectangle {
                    id: frontrect
                    color: "transparent"
                    anchors.fill: parent

                    UbuntuShape {
                        id: currentCondition
                        color: "#10000000"
                        radius: "medium"

                        width: units.gu(15)
                        height: units.gu(15)
                        anchors {
                            horizontalCenter: parent.horizontalCenter
                            horizontalCenterOffset: units.gu(-8)
                            verticalCenter: parent.verticalCenter
                        }

                        Components.WeatherConditionIconComponent {
                            id: currentConditionIcon
                            anchors.centerIn: parent
                            anchors.margins: units.gu(1)
                            condition: currentWeather.icon
                        }
                    }

                    UbuntuShape {
                        id: currentConditionTemp
                        color: "#10000000"
                        radius: "medium"
                        width: units.gu(15)
                        height: units.gu(15)
                        anchors {
                            horizontalCenter: parent.horizontalCenter
                            horizontalCenterOffset: units.gu(8)
                            verticalCenter: parent.verticalCenter
                        }

                        Components.WeatherTemperatureComponent {
                            id: currentConditionTempContent
                            currentTemp: currentWeather.currentTemp
                            tempScale: currentWeather.tempScale
                            minTemp: currentWeather.minTemp
                            maxTemp: currentWeather.maxTemp
                            anchors.fill: parent
                        }
                    }
                }

            states:[
                State {
                    name: "back"
                    when: flipable.flipped
                },
                State {
                    name: "front"
                    when: !flipable.flipped
                }
            ]

            transitions:[
                Transition {
                    to:"back"
                    SequentialAnimation{
                        ParallelAnimation {
                            NumberAnimation { target: currentCondition; property: "anchors.verticalCenterOffset"; duration: UbuntuAnimation.SlowDuration; to: units.gu(8); easing: UbuntuAnimation.StandardEasing }
                            NumberAnimation { target: currentConditionTemp; property: "anchors.verticalCenterOffset"; duration: UbuntuAnimation.SlowDuration; to: units.gu(-8); easing: UbuntuAnimation.StandardEasing }
                            NumberAnimation {target: currentConditionIcon; property: "opacity"; to:0; duration: UbuntuAnimation.SlowDuration}
                            NumberAnimation {target: currentConditionTempContent; property: "opacity"; to:0; duration: UbuntuAnimation.SlowDuration}
                        }
                        ParallelAnimation {
                            NumberAnimation {target: backrect; property: "opacity"; to:1; duration: UbuntuAnimation.SlowDuration}
                            NumberAnimation {target: frontrect; property: "opacity"; to:0; duration: UbuntuAnimation.SlowDuration}
                        }
                    }
                },
                Transition {
                    from: "back"
                    to:"front"
                    ScriptAction{
                        script: {
                            if(dailyForecastList.currentItem == dailyForecastItem){
                                 back2frontAnim.start();
                            }else{
                                backrect.opacity = 0;
                                frontrect.opacity = 1;
                                currentConditionIcon.opacity = 1;
                                currentConditionTempContent.opacity = 1;
                                currentCondition.anchors.verticalCenterOffset = 0;
                                currentConditionTemp.anchors.verticalCenterOffset = 0;
                            }
                        }
                    }
                }
            ]

            SequentialAnimation{
                id: back2frontAnim
                running: false
                ParallelAnimation {
                    NumberAnimation {target: frontrect; property: "opacity"; to:1; duration: UbuntuAnimation.SlowDuration}
                    NumberAnimation {target: backrect; property: "opacity"; to:0; duration: UbuntuAnimation.SlowDuration}
                }
                ParallelAnimation{
                    NumberAnimation {target: currentConditionIcon; property: "opacity"; to:1; duration: UbuntuAnimation.SlowDuration}
                    NumberAnimation {target: currentConditionTempContent; property: "opacity"; to:1; duration: UbuntuAnimation.SlowDuration}
                    NumberAnimation { target: currentCondition; property: "anchors.verticalCenterOffset"; duration: UbuntuAnimation.SlowDuration; to: 0; easing: UbuntuAnimation.StandardEasingReverse }
                    NumberAnimation { target: currentConditionTemp; property: "anchors.verticalCenterOffset"; duration: UbuntuAnimation.SlowDuration; to: 0; easing: UbuntuAnimation.StandardEasingReverse }
                }
            }
        }
    }

