/*
 * Copyright (C) 2013 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Raúl Yeguas <neokore@gmail.com>
 *              Martin Borho <martin@borho.net>
 */
import QtQuick 2.0
import Ubuntu.Components.Popups 0.1
import Ubuntu.Components 0.1
import QtFeedback 5.0

Tab {
    id: locationTab
    objectName: "LocationTab"

    anchors.fill: parent

    property int locationIndex: 0
    property var locationData: {}

    function formatTimestamp(dateData, format) {
        var date = new Date(dateData.year, dateData.month, dateData.date, dateData.hours, dateData.minutes)
        return Qt.formatDate(date, format)
    }

    onVisibleChanged: {
        if(dailyForecastList.currentIndex > -1) {
            dailyForecastList.currentIndex = 0;
            if(visible == true) {
                var data = dayForecastModel.get(0)
                adjustBackground(Math.round(data.temp))
            }
        }
    }

    Component.onCompleted: {
        var locData = mainView.locationsList[locationIndex],
            units = (mainView.settings.units === 'imperial') ? 'imperial' : 'metric',
            wind_units = (mainView.settings.wind_units === 'mph') ? 'imperial' : 'metric',
            precip_units = (mainView.settings.precip_units === 'in') ? 'imperial' : 'metric',
            dailyForecasts = locData.data,
            dailyLength = (dailyForecasts) ? dailyForecasts.length : 0;

        // set location data as property
        locationData = locData;

        // set last updated label
        lastUpdatedComponent.updateDate = (locationData.updated) ? locationData.updated : new Date().getTime()

        // set daily forecasts
        if(dailyLength > 0) {
            noDataAvaible.visible = false;
            for(var x=0;x<dailyLength;x++) {
                //
                dayForecastModel.append({
                    dateRel: "",//Tomorrow",
                    date: formatTimestamp(dailyForecasts[x].date, 'dddd, dd MMMM'),
                    temp: (dailyForecasts[x]["current"] === undefined) ? dailyForecasts[x][units].tempMax :
                                                dailyForecasts[x]["current"][units].temp,
                    tempMin: dailyForecasts[x][units].tempMin,
                    tempMax: (dailyForecasts[x]["current"] !== undefined) ? dailyForecasts[x][units].tempMax :
                                                null,
                    cond: dailyForecasts[x].condition.id,
                    condIcon: dailyForecasts[x].icon,
                    wind_speed: dailyForecasts[x][wind_units].windSpeed,
                    wind_dir: dailyForecasts[x].windDir,
                    humid: dailyForecasts[x].humidity,
                    precip: +(dailyForecasts[x][precip_units].rain || dailyForecasts[x][precip_units].snow || 0).toFixed(2),
                    press: dailyForecasts[x].pressure,
                    hours: dailyForecasts[x]["hourly"]
                });
            }
        } else {
            noDataAvaible.visible = true;
        }
    }

    HapticsEffect {
        id: hourlyScrollEffect
        attackTime: 1
        intensity: 0.2
        duration: 5
        fadeTime: 1
    }

    // Menu for options
    page: Page {
        id: locationPage
        isLeaf: true
        anchors.fill: parent
        anchors.topMargin: units.gu(9.5)

        ListModel {
            id: dayForecastModel
        }

        ListView {
            id: dailyForecastList
            width: parent.width
            height:parent.height
            model:dayForecastModel

            onCurrentIndexChanged: {
                if(visible) {
                    var data = dayForecastModel.get(currentIndex)
                    adjustBackground(Math.round(data.temp))
                    dailyForecastList.currentItem.appearAnimation.start()
                }
            }

            interactive: false
            snapMode: ListView.SnapOneItem
            highlightRangeMode: ListView.StrictlyEnforceRange
            verticalLayoutDirection: ListView.TopToBottom
            flickableDirection: Flickable.VerticalFlick
            clip: true

            delegate: Item {
                id: dailyForecastItem
                width:parent.width
                height: locationPage.height

                property NumberAnimation appearAnimation: currentWeatherComponent.appearAnimation

                Rectangle {
                    id: listRectangle
                    width: parent.width
                    height: dailyForecastList.height
                    anchors.verticalCenter: parent.verticalCenter
                    color: "transparent"

                    DateComponent {
                        id: dateComponent
                        dateRelative: dateRel
                        dateString: date
                        borderTopVisible: false
                    }
                    CurrentWeather {
                        id: currentWeatherComponent
                        currentTemp: temp
                        minTemp: tempMin
                        maxTemp: tempMax
                        icon: condIcon
                        condition: cond
                        windSpeed: wind_speed
                        windDir: wind_dir
                        humidity: humid
                        precipitation: precip
                        pressure: press
                        hourly: hours
                        anchors.top: dateComponent.bottom
                        showMaxTemp: (index === 0)
                    }
                }
            }
            LastUpdatedComponent{
                id: lastUpdatedComponent
            }
            Rectangle {
                id: noDataAvaible
                visible: false
                width:parent.width-units.gu(10)
                anchors.centerIn:parent
                anchors.verticalCenterOffset: -units.gu(10)
                color: "transparent"
                Label {
                    width:parent.width
                    text: i18n.tr("No weather data available at the moment, please try to refresh later again!")
                    fontSize: "large"
                    wrapMode: Text.WordWrap
                }
            }
            Image {
                id: dataProviderLogo
                source: Qt.resolvedUrl("../resources/images/openWeatherMapLogo.png")
                anchors {
                    bottom: parent.bottom
                    right: parent.right
                    bottomMargin: units.gu(1)
                    rightMargin: units.gu(1)
                }
                height: units.gu(8)
                width: units.gu(8)
                fillMode: Image.PreserveAspectFit
                MouseArea {
                    anchors.fill: parent
                    onClicked: Qt.openUrlExternally("http://m.openweathermap.org/city/" + locationData.location.services.openweathermap)
                }
            }
        }


        tools: ToolbarItems {
            id: pageTools
            ToolbarButton {
                objectName: "RefreshButton"
                action:Action {
                    id: refreshAction
                    objectName: "RefreshAction"
                    iconSource: Qt.resolvedUrl("../resources/images/refresh_icon.png")
                    text: i18n.tr("Refresh")

                    onTriggered: {
                        mainView.tabIndexAtRefresh = tabs.selectedTabIndex
                        mainView.refreshData(false, true);
                    }
                }
            }
            ToolbarButton {
                objectName: "EditButton"
                action:Action {
                    id: editLocationAction
                    iconSource: Qt.resolvedUrl("../resources/images/add_icon.png")
                    text: i18n.tr("Edit")

                    onTriggered: {
                        mainView.showLocationManager()
                    }
                }
            }
            ToolbarButton {
                objectName: "SettingsButton"
                action:Action {
                    id: configAction
                    iconSource: Qt.resolvedUrl("../resources/images/settings.png")
                    text: i18n.tr("Settings")

                    onTriggered: {
                        mainView.showSettings();
                    }
                }
            }
        }
    }
}
